import { Associative, KeyedMapFn } from '@collectable/core';
import { RedBlackTree, RedBlackTreeEntry } from '../internals';
/**
 * Returns an array of key/value tuples. Keys appear first in each tuple, followed by the associated value in the tree.
 * The array is guaranteed to be in the same order as the corresponding entries in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {RedBlackTreeEntry<K, V>[]} An array of key/value pairs from the tree
 */
export declare function arrayFrom<K, V>(tree: RedBlackTree<K, V>): RedBlackTreeEntry<K, V>[];
/**
 * Maps the contents of the tree to an array of transformed values. The array is guaranteed to be in the same order as
 * the corresponding entries in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @template U The type of value that will be populated into the returned array
 * @param {KeyedMapFn<K, V, U>} mapper A callback function that maps an entry in the tree to a new value
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {U[]} An array of transformed values; one for each entry in the tree
 */
export declare function arrayFrom<K, V, U>(mapper: KeyedMapFn<K, V, U>, tree: RedBlackTree<K, V>): U[];
/**
 * Returns a value iterator; one for each entry in the tree. The iterator is guaranteed to iterate in the same order as
 * the corresponding entries in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {IterableIterator<V>} An iterable iterator that will visit each value in the tree
 */
export declare function values<K, V>(tree: RedBlackTree<K, V>): IterableIterator<V>;
/**
 * Returns a key iterator; one for each entry in the tree. The iterator is guaranteed to iterate in the same order as
 * the corresponding entries in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {IterableIterator<K>} An iterable iterator that will visit each key in the tree
 */
export declare function keys<K, V>(tree: RedBlackTree<K, V>): IterableIterator<K>;
/**
 * Returns the tree unwrapped as a plain JavaScript object. Keys are treated as (or converted to) strings.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {boolean} deep If true, any valid Collectable.js collection values nested in the tree will also be unwrapped
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {Associative<V>} A plain JavaScript object containing entries from the tree.
 */
export declare function unwrap<K, V>(deep: boolean, tree: RedBlackTree<K, V>): Associative<V>;
